package com.shanghaichengdi.downloadcenterserver.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.shanghaichengdi.downloadcenterserver.domain.BoatWarning;
import com.shanghaichengdi.downloadcenterserver.domain.BoatWarningCxDict;
import com.shanghaichengdi.downloadcenterserver.domain.ManningCertRealtime;
import com.shanghaichengdi.downloadcenterserver.domain.MvwNationalIssInfoRealtime;
import com.shanghaichengdi.downloadcenterserver.enums.WarningType;
import com.shanghaichengdi.downloadcenterserver.mapper.ManningCertRealtimeMapper;
import com.shanghaichengdi.downloadcenterserver.mapper.NationalIssInfoRealtimeMapper;
import com.shanghaichengdi.downloadcenterserver.service.BoatWarningCxDictService;
import com.shanghaichengdi.downloadcenterserver.service.BoatWarningService;
import com.shanghaichengdi.downloadcenterserver.mapper.BoatWarningMapper;
import com.shanghaichengdi.downloadcenterserver.util.PrintLogUtils;
import com.shanghaichengdi.downloadcenterserver.util.SnowflakeMakeConf;
import com.shanghaichengdi.downloadcenterserver.vo.ShipIdentificationVo;
import com.shanghaigeography.Util.DateUtils;
import java.text.ParseException;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

/**
 * @author 14390
 * @description 针对表【BOAT_WARNING(船舶预警)】的数据库操作Service实现
 * @createDate 2024-08-28 16:39:41
 */
@Slf4j
@Service
public class BoatWarningServiceImpl extends ServiceImpl<BoatWarningMapper, BoatWarning>
    implements BoatWarningService {

  @Resource
  BoatWarningMapper mapper;
  @Resource
  ManningCertRealtimeMapper manningCertRealtimeMapper;
  @Resource
  NationalIssInfoRealtimeMapper nationalIssInfoRealtimeMapper;
  @Resource
  SnowflakeMakeConf snowflakeConfig;
  @Resource
  BoatWarningCxDictService boatWarningCxDictService;

  ExecutorService executorService = Executors.newFixedThreadPool(6);
  private static Integer zxTotal = 0;
  private static Integer cxTotal = 0;

  @Override
  public void cxExecute() {

  }

  @Override
  public Integer zsExecute() throws ParseException {

    List<ShipIdentificationVo> voList = mapper.selectBoatData();
    log.info("船舶通航感知的数据大小为:{}", voList.size());
    List<BoatWarningCxDict> cxDictList = boatWarningCxDictService.list();
    log.info("超限字典数据为:{}", JSONObject.toJSONString(cxDictList));
    for (ShipIdentificationVo vo : voList) {
      String name;
      if (StringUtils.hasText(vo.getAnalysisName())) {
        name = vo.getAnalysisName();
      } else if (StringUtils.hasText(vo.getBoatName())) {
        name = vo.getBoatName();
      } else {
        name = "";
        continue;
      }
      executorService.execute(() -> {
        try {
          if (StringUtils.hasLength(name)) {
            generateZsWarning1(vo, name);
            generateZsWarning2(vo, name);
//            generateCxWarning(vo, cxDictList);
          }
        } catch (Exception e) {
          PrintLogUtils.printErrorLogInfo(e);
        }
      });
    }
    log.warn("共计生成:{} 条证书数据，  {} 条超限数据", zxTotal, cxTotal);
    return zxTotal;
  }

  private void generateCxWarning(ShipIdentificationVo vo, List<BoatWarningCxDict> cxDictList)
      throws Exception {
    for (BoatWarningCxDict cxDict : cxDictList) {
      if (cxDict.getName().equals(vo.getBayonetName()) && !Objects.isNull(vo.getLength())
          && !Objects.isNull(vo.getWidth())) {
        if (vo.getLength() > cxDict.getCcLimit() || vo.getWidth() > cxDict.getCkLimit()) {
          generateBoatWarning(WarningType.CX, vo, "超限。");
          cxTotal++;
        }
      }
    }
  }

  private void generateZsWarning2(ShipIdentificationVo vo, String name) throws ParseException {
    List<MvwNationalIssInfoRealtime> issInfoRealtimeList = nationalIssInfoRealtimeMapper.selectData(
        name);
    if (!CollectionUtils.isEmpty(issInfoRealtimeList)) {
      MvwNationalIssInfoRealtime issInfoRealtime = issInfoRealtimeList.stream()
          .sorted(Comparator.comparing(MvwNationalIssInfoRealtime::getLastUpdateDate).reversed())
          .collect(Collectors.toList()).get(0);
      if (issInfoRealtime.getExpdateEnd().compareTo(new Date()) < 0) {
        log.info("产生证书预警数据,船舶国籍证书过期具体信息为:{}",
            JSONObject.toJSONString(issInfoRealtime));
        generateBoatWarning(WarningType.ZS, vo, "船舶国籍证书过期");
        zxTotal++;
      }
    }
  }

  private void generateZsWarning1(ShipIdentificationVo vo, String name) throws ParseException {
    List<ManningCertRealtime> manningCertRealtimeList = manningCertRealtimeMapper.selectList(
        Wrappers.<ManningCertRealtime>lambdaQuery().eq(ManningCertRealtime::getShipNameCn, name));
    if (!CollectionUtils.isEmpty(manningCertRealtimeList)) {
      ManningCertRealtime certRealtime = manningCertRealtimeList.stream()
          .sorted(Comparator.comparing(ManningCertRealtime::getLastUpdateDate).reversed())
          .collect(Collectors.toList()).get(0);
      if (certRealtime.getExpdateEnd().compareTo(new Date()) < 0) {
        log.info("产生证书预警数据,船舶最低安全配员_配员证书过期具体信息为:{}",
            JSONObject.toJSONString(certRealtime));
        generateBoatWarning(WarningType.ZS, vo, "船舶最低安全配员_配员证书过期");
        zxTotal++;
      }
    }
  }


  public void generateBoatWarning(WarningType warningType, ShipIdentificationVo data, String detail)
      throws ParseException {
    BoatWarning boatWarning = new BoatWarning();
    boatWarning.setWarningType(warningType.getTypeCode());
    boatWarning.setCode(snowflakeConfig.snowflakeStringId());
    boatWarning.setEventcode(data.getCode());
    boatWarning.setWarncode(
        warningType.getLogo() + DateUtils.dateToString(data.getAnalysisTime(),
            DateUtils.DATE_FORMAT_14)
            + new Random().nextInt(100000));
    boatWarning.setWarningTime(data.getAnalysisTime());
    boatWarning.setBoatName(
        Objects.isNull(data.getAnalysisName()) ? data.getBoatName() : data.getAnalysisName());
    boatWarning.setPassTime(data.getAnalysisTime());
    boatWarning.setWarningDetail(
        "名称为:" + boatWarning.getBoatName() + "的船舶在进入卡口区域疑似" + detail + "。");
    super.save(boatWarning);
  }

  public static void main(String[] args) {
    Date date = new Date(1621612800000l);
    System.out.println(new Random().nextInt(100000));
  }
}




