package com.shanghaichengdi.downloadcenterserver.util;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;

/**
 * @author lag
 */
public class HTTPUtil {
    /**
     * 向指定 URL 发送POST方法的请求
     *
     * @param url
     *            发送请求的 URL
     * @param param
     *            请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return 所代表远程资源的响应结果
     */
    public static String sendPost(String url, String param) throws RuntimeException {
        PrintWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 获取URLConnection对象对应的输出流
            out = new PrintWriter(conn.getOutputStream());
            // 发送请求参数
            out.print(new String(param.getBytes(),"utf-8"));
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(
                    new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            System.out.println("发送 POST 请求出现异常！"+e);
            e.printStackTrace();
            throw new RuntimeException(e.getMessage());
        }
        //使用finally块来关闭输出流、输入流
        finally{
            try{
                if(out!=null){
                    out.close();
                }
                if(in!=null){
                    in.close();
                }
            }
            catch(IOException ex){
                ex.printStackTrace();
            }
        }
        return result;
    }

    public static void save(String url, String param) throws IOException {
        System.out.println("===   访问:"+url);
        URL urls = new URL(url);
        HttpURLConnection conn = null;
        BufferedReader in=null;
        PrintWriter out=null;
        String result="";
        try {
            conn = (HttpURLConnection) urls.openConnection();
            conn.setRequestMethod("POST");
            conn.setDoOutput(true); // post发送数据时必须设置为true，否则对方无法接收到数据
            conn.connect();
            out = new PrintWriter(new OutputStreamWriter(conn.getOutputStream(),"utf-8"));
//            // 发送请求参数
            out.print(param);
            // flush输出流的缓冲
            out.flush();
            System.out.println(conn.getResponseCode());
            in = new BufferedReader(new InputStreamReader(conn.getInputStream(),"utf-8"));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
            conn.getOutputStream().flush();
            conn.getOutputStream().close();
        } catch (Exception ex) {
            //todo something
        } finally {
            conn.disconnect();
            if (in !=null) {
                in.close();
            }
            if (out!=null)
            {
                out.close();
            }
        }
        System.out.println(result);
    }

    public static String httpClientPost(String urlParam, String str, String charset) {
    StringBuffer resultBuffer = null;
    HttpClient client = new DefaultHttpClient();
    HttpPost httpPost = new HttpPost(urlParam);
    BufferedReader br = null;
    try {
      StringEntity s = new StringEntity(str, charset);
      s.setContentEncoding("UTF-8");
      s.setContentType("application/json");
      httpPost.setEntity(s);
      // 读取服务器响应数据
      resultBuffer = new StringBuffer();
      HttpResponse response = client.execute(httpPost);
      br = new BufferedReader(new InputStreamReader(response.getEntity().getContent(), charset));
      String temp;
      while ((temp = br.readLine()) != null) {
        if (temp.length() > 1024 * 1024) {
          throw new RuntimeException("文件单行长度过长");
        }
        resultBuffer.append(temp + "\n");
      }
    } catch (Exception e) {
      throw new RuntimeException(e);
    } finally {
      if (br != null) {
        try {
          br.close();
        } catch (IOException e) {
          br = null;
          throw new RuntimeException(e);
        }
      }
    }
    return resultBuffer.toString();
  }
}
